import { Ref } from 'vue-demi';
import { Ref as Ref$1 } from '@vue/composition-api';

declare const useDebouncedRef: (value: any, delay?: number) => Ref;

declare type Fn = () => void;

/**
 * Call onMounted() if it's inside a component lifecycle, if not, run just call the function
 * @param fn
 * @param sync if set to false, it will run in the nextTick() of Vue
 */
declare function tryOnMounted(fn: Fn, sync?: boolean): void;
/**
 * Call onUnmounted() if it's inside a component lifecycle, if not, do nothing
 * @param fn
 */
declare function tryOnUnmounted(fn: Fn): void;

declare function useReactiveState(getter: () => Required<object>): [Required<object>, VoidFunction];

/**
 * Reactive Resize API, When window resize, do someting.
 * @param {function} fn
 * @param {boolean} useCapture
 * @param {boolean} isThrottle: true: throttle | false: debounce
 * @return {function} dispatchResize
 */
declare const useResize: (fn: Fn, useCapture?: boolean, isThrottle?: boolean) => {
    dispatchResize: () => void;
};

/**
 * Reactive Fullscreen API build on top of screenfull.js
 * @param {HTMLElement} target
 * @return {boolean} isFullscreen
 * @return {function} toggle
 */
declare const useFullscreen: (target: HTMLElement) => {
    isFullscreen: Ref$1<boolean>;
    toggleFullscreen: () => void;
};

export { tryOnMounted, tryOnUnmounted, useDebouncedRef, useFullscreen, useReactiveState, useResize };
