(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('vue-demi'), require('lodash'), require('screenfull')) :
  typeof define === 'function' && define.amd ? define(['exports', 'vue-demi', 'lodash', 'screenfull'], factory) :
  (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory(global.VueUse = {}, global.VueDemi, global.lodash, global.screenfull));
}(this, (function (exports, vueDemi, lodash, screenfull) { 'use strict';

  function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

  var screenfull__default = /*#__PURE__*/_interopDefaultLegacy(screenfull);

  const useDebouncedRef = (value, delay = 200) => {
      let timeout;
      return vueDemi.customRef((track, trigger) => {
          return {
              get() {
                  track();
                  return value;
              },
              set(newValue) {
                  clearTimeout(timeout);
                  timeout = setTimeout(() => {
                      value = newValue;
                      trigger();
                  }, delay);
              }
          };
      });
  };

  /**
   * Call onMounted() if it's inside a component lifecycle, if not, run just call the function
   * @param fn
   * @param sync if set to false, it will run in the nextTick() of Vue
   */
  function tryOnMounted(fn, sync = true) {
      if (vueDemi.getCurrentInstance()) {
          vueDemi.onMounted(fn);
      }
      else if (sync) {
          fn();
      }
      else {
          vueDemi.nextTick(fn);
      }
  }
  /**
   * Call onUnmounted() if it's inside a component lifecycle, if not, do nothing
   * @param fn
   */
  function tryOnUnmounted(fn) {
      if (vueDemi.getCurrentInstance())
          vueDemi.onUnmounted(fn);
  }

  function useReactiveState(getter) {
      const state = vueDemi.reactive(getter());
      const update = () => {
          Object.assign(state, getter());
      };
      return [state, update];
  }

  /**
   * Reactive Resize API, When window resize, do someting.
   * @param {function} fn
   * @param {boolean} useCapture
   * @param {boolean} isThrottle: true: throttle | false: debounce
   * @return {function} dispatchResize
   */
  const useResize = (fn, useCapture = false, isThrottle = false) => {
      const dispatchResize = () => {
          const e = document.createEvent('Event');
          e.initEvent('resize', true, true);
          window.dispatchEvent(e);
      };
      const waitFn = isThrottle ? lodash.throttle(fn, 500) : lodash.debounce(fn, 500);
      tryOnMounted(() => {
          window.addEventListener('resize', waitFn, useCapture);
      });
      tryOnUnmounted(() => {
          window.removeEventListener('resize', waitFn, useCapture);
      });
      return {
          dispatchResize
      };
  };

  /**
   * Reactive Fullscreen API build on top of screenfull.js
   * @param {HTMLElement} target
   * @return {boolean} isFullscreen
   * @return {function} toggle
   */
  const useFullscreen = (target) => {
      const targetRef = vueDemi.ref(target || document?.querySelector('html'));
      const isFullscreen = vueDemi.ref(false);
      const toggleFullscreen = () => {
          screenfull__default['default'].isEnabled && screenfull__default['default'].toggle(targetRef.value);
      };
      tryOnMounted(() => {
          if (screenfull__default['default'].isEnabled) {
              screenfull__default['default'].on('change', () => {
                  isFullscreen.value = screenfull__default['default'].isFullscreen;
              });
          }
      });
      return {
          isFullscreen,
          toggleFullscreen
      };
  };

  exports.tryOnMounted = tryOnMounted;
  exports.tryOnUnmounted = tryOnUnmounted;
  exports.useDebouncedRef = useDebouncedRef;
  exports.useFullscreen = useFullscreen;
  exports.useReactiveState = useReactiveState;
  exports.useResize = useResize;

  Object.defineProperty(exports, '__esModule', { value: true });

})));
